@@ -24,7 +24,7 @@ from . import (
     users,
     workflows,
 )
-from .auth import TokenAuth
+from .auth import BasicAuth, TokenAuth
 from .compat import text
 from .constants import (
     CHUNK_SIZE,
@@ -32,7 +32,6 @@ from .constants import (
     DEFAULT_APP_NAME,
     DEFAULT_URL,
     IDEMPOTENCY_KEY,
-    LOG_LIMIT_SIZE,
     MAX_RETRY,
     RETRY_BACKOFF_FACTOR,
     RETRY_METHODS,
@@ -49,7 +48,7 @@ from .exceptions import (
     Unauthorized,
 )
 from .tcp import TCPKeepAliveHTTPSAdapter
-from .utils import get_response_content, json_helper
+from .utils import json_helper, log_response
 
 try:
     from typing import TYPE_CHECKING
@@ -110,7 +109,7 @@ class NuxeoClient(object):
         **kwargs  # type: Any
     ):
         # type: (...) -> None
-        self.auth = auth
+        self.auth = BasicAuth(*auth) if isinstance(auth, tuple) else auth
         self.host = host
         self.api_path = api_path
         self.chunk_size = chunk_size
@@ -126,6 +125,7 @@ class NuxeoClient(object):
         self.schemas = kwargs.get("schemas", "*")
         self.repository = kwargs.pop("repository", "default")
         self._session = requests.sessions.Session()
+        self._session.hooks["response"] = [log_response]
         cookies = kwargs.pop("cookies", None)
         if cookies:
             self._session.cookies = cookies
@@ -282,6 +282,9 @@ class NuxeoClient(object):
         # to set `default` to `None`.
         default = kwargs.pop("default", object)
 
+        # Allow to pass a custom authentication class
+        auth = kwargs.pop("auth", None) or self.auth
+
         _kwargs = {k: v for k, v in kwargs.items() if k != "params"}
         logger.debug(
             (
@@ -299,9 +302,8 @@ class NuxeoClient(object):
         exc = None
         try:
             resp = self._session.request(
-                method, url, headers=headers, auth=self.auth, data=data, **kwargs
+                method, url, headers=headers, auth=auth, data=data, **kwargs
             )
-            self._log_response(resp)
             resp.raise_for_status()
         except Exception as exc:
             if default is object:
@@ -341,6 +343,7 @@ class NuxeoClient(object):
         # type: (...) -> Text
         """
         Request a token for the user.
+        It should only be used if you want to get a Nuxeo token from a Basic Auth.
 
         :param device_id: device identifier
         :param permission: read/write permissions
@@ -348,22 +351,20 @@ class NuxeoClient(object):
         :param device: optional device description
         :param revoke: revoke the token
         """
-
-        parameters = {
-            "deviceId": device_id,
-            "applicationName": app_name,
-            "permission": permission,
-            "revoke": text(revoke).lower(),
-        }
-        if device:
-            parameters["deviceDescription"] = device
-
-        path = "authentication/token"
-        token = self.request("GET", path, params=parameters).text
+        auth = TokenAuth("")
+        token = auth.request_token(
+            self,
+            device_id,
+            permission,
+            app_name=app_name,
+            device=device,
+            revoke=revoke,
+            auth=self.auth,
+        )
 
         # Use the (potentially re-newed) token from now on
         if not revoke:
-            self.auth = TokenAuth(token)
+            self.auth = auth
         return token
 
     def is_reachable(self):
@@ -412,75 +413,23 @@ class NuxeoClient(object):
         if not isinstance(error, requests.HTTPError):
             return error
 
+        response = error.response
+        error_data = {}
         try:
-            error_data = error.response.json()
+            error_data.update(response.json())
         except ValueError:
-            error_data = {
-                "status": error.response.status_code,
-                "message": error.response.content,
-            }
+            error_data["message"] = response.content
+        finally:
+            error_data["status"] = response.status_code
+            if not error_data.get("message", ""):
+                error_data["message"] = response.reason
 
-        status = error_data.get("status", 0)
-        request_uid = error.response.headers.get(IDEMPOTENCY_KEY, "")
+        status = error_data["status"]
+        request_uid = response.headers.get(IDEMPOTENCY_KEY, "")
         if status == 409 and request_uid:
             return OngoingRequestError(request_uid)
         return HTTP_ERROR.get(status, HTTPError).parse(error_data)
 
-    @staticmethod
-    def _log_response(response, limit_size=LOG_LIMIT_SIZE):
-        # type: (requests.Response, int) -> None
-        """
-        Log the server's response based on its content type.
-
-        :param response: The server's response to handle
-        :param limit_size: Maximum size to not overflow when printing raw content
-        of the response
-        """
-
-        # No need to do more work if nobody will see it
-        if logger.getEffectiveLevel() > logging.DEBUG:
-            return
-
-        headers = response.headers
-        content_type = headers.get("content-type", "application/octet-stream")
-        content_size = int(headers.get("content-length", 0))
-        chunked = headers.get("transfer-encoding", "") == "chunked"
-
-        if response.status_code and response.status_code >= 400:
-            # This is a request ending on an error
-            content = get_response_content(response, limit_size)
-        if response.url.endswith("site/automation"):
-            # This endpoint returns too many information and pollute logs.
-            # Besides contents of this call are stored into the .operations attr.
-            content = "<Automation details saved into the *operations* attr>"
-        elif response.url.endswith("json/cmis"):
-            # This endpoint returns too many information and pollute logs.
-            # Besides contents of this call are stored into the .server_info attr.
-            content = "<CMIS details saved into the *server_info* attr>"
-        elif (
-            not content_type.startswith("text")
-            and "json" not in content_type
-            and content_size
-        ):
-            # The Content-Type is a binary one, but it does not contain JSON data
-            # Skipped binary types are everything but "text/xxx":
-            #   https://www.iana.org/assignments/media-types/media-types.xhtml
-            content = "<binary data ({:,} bytes)>".format(content_size)
-        elif chunked or content_size > 0:
-            # At this point, we should only have text data not bigger than *limit_size*.
-            content = get_response_content(response, limit_size)
-        else:
-            # response.content is empty when *void_op* is True,
-            # meaning we do not want to get back what we sent
-            # or the operation does not return anything by default
-            content = "<no content>"
-
-        logger.debug(
-            "Response from {!r} [{}]: {!r} with headers {!r} and cookies {!r}".format(
-                response.url, response.status_code, content, headers, response.cookies
-            )
-        )
-
 
 class Nuxeo(object):
     """
