@@ -123,6 +123,17 @@ class InvalidUploadHandler(NuxeoError):
         return repr(self)
 
 
+class OAuth2Error(HTTPError):
+    """ Exception thrown when an OAuth2 error happens. """
+
+    status = 400
+
+    def __init__(self, error):
+        # type: (Text) -> None
+        self.message = error
+        self.stacktrace = None
+
+
 class OngoingRequestError(Conflict):
     """ Exception thrown when doing an idempotent call that is already being processed. """
 
