@@ -14,16 +14,7 @@ try:
 
     if TYPE_CHECKING:
         from typing import Any, BinaryIO, Dict, List, Optional, Text, Union
-        from .comments import API as CommentsAPI
-        from .directories import API as DirectoriesAPI
-        from .documents import API as DocumentsAPI
         from .endpoint import APIEndpoint
-        from .groups import API as GroupsAPI
-        from .operations import API as OperationsAPI
-        from .tasks import API as TasksAPI
-        from .uploads import API as UploadsAPI
-        from .users import API as UsersAPI
-        from .workflows import API as WorkflowsAPI
 except ImportError:
     pass
 
@@ -34,24 +25,27 @@ except ImportError:
 class Model(object):
     """ Base class for all entities. """
 
-    _valid_properties = {}  # type: Dict[Text, Any]
-    service = None  # type: APIEndpoint
-    uid = None  # type: Text
+    __slots__ = {"service": None}  # type: Dict[Text, Any]
 
     def __init__(self, service=None, **kwargs):
         # type: (Optional[APIEndpoint], Any) -> None
-        self.service = service
+        self.service = service  # type: APIEndpoint
 
         # Declare attributes
-        for key, default in type(self)._valid_properties.items():
+        for key, default in type(self).__slots__.copy().items():
+            # Reset mutable objects to prevent data leaks
+            if isinstance(default, dict):
+                default = {}
+            elif isinstance(default, list):
+                default = []
             key = key.replace("-", "_")
             setattr(self, key, kwargs.get(key, default))
 
     def __repr__(self):
         # type: () -> Text
         attrs = ", ".join(
-            "{}={!r}".format(attr, getattr(self, attr.replace("-", "_"), None))
-            for attr in sorted(self._valid_properties)
+            "{}={!r}".format(attr.replace("_", "-"), getattr(self, attr, None))
+            for attr in sorted(self.__slots__)
         )
         return "<{} {}>".format(self.__class__.__name__, attrs)
 
@@ -59,15 +53,15 @@ class Model(object):
         # type: () -> Dict[Text, Any]
         """ Returns a dict representation of the resource. """
         result = {}
-        for key in self._valid_properties:
-            val = getattr(self, key.replace("-", "_"))
+        for key in self.__slots__:
+            val = getattr(self, key)
             if val is None:
                 continue
             # Parse lists of objects
             elif isinstance(val, list) and len(val) > 0 and isinstance(val[0], Model):
                 val = [item.as_dict() for item in val]
 
-            result[key] = val
+            result[key.replace("_", "-")] = val
         return result
 
     @classmethod
@@ -84,6 +78,9 @@ class Model(object):
 
 
 class RefreshableModel(Model):
+
+    __slots__ = ()
+
     def load(self, model=None):
         # type: (Optional[Union[Model, Dict[Text, Any]]]) -> None
         """
@@ -108,8 +105,7 @@ class RefreshableModel(Model):
             def get_prop(obj, key):
                 return getattr(obj, key)
 
-        for key in self._valid_properties:
-            key = key.replace("-", "_")
+        for key in self.__slots__.keys():
             setattr(self, key, get_prop(model, key))
 
 
@@ -119,7 +115,7 @@ class RefreshableModel(Model):
 class Batch(Model):
     """ Upload batch. """
 
-    _valid_properties = {
+    __slots__ = {
         "batchId": None,
         "blobs": {},
         "dropped": None,
@@ -130,7 +126,6 @@ class Batch(Model):
         "provider": None,
         "upload_idx": 0,
     }
-    service = None  # type: UploadsAPI
 
     @property
     def uid(self):
@@ -234,7 +229,7 @@ class Batch(Model):
 class Blob(Model):
     """ Blob superclass used for metadata. """
 
-    _valid_properties = {
+    __slots__ = {
         "batchId": "",
         "chunkCount": 0,
         "fileIdx": None,
@@ -246,7 +241,6 @@ class Blob(Model):
         "uploadedChunkIds": [],
         "uploadedSize": 0,
     }
-    service = None  # type: UploadsAPI
 
     @classmethod
     def parse(cls, json, service=None):
@@ -273,8 +267,6 @@ class BufferBlob(Blob):
     with the `with` statement.
     """
 
-    stringio = None  # type: Optional[StringIO]
-
     def __init__(self, data, **kwargs):
         # type: (Text, Any) -> None
         """
@@ -282,6 +274,7 @@ class BufferBlob(Blob):
         :param **kwargs: named attributes
         """
         super(BufferBlob, self).__init__(**kwargs)
+        self.stringio = None  # type: Optional[StringIO]
         self.buffer = data
         self.size = len(self.buffer)
         self.mimetype = "application/octet-stream"
@@ -306,12 +299,12 @@ class BufferBlob(Blob):
 class Comment(Model):
     """ Comment. """
 
-    _valid_properties = {
+    __slots__ = {
         "ancestorIds": [],
         "author": None,
         "creationDate": None,
         "entity": None,
-        "entity-type": "comment",
+        "entity_type": "comment",
         "entityId": None,
         "id": None,
         "lastReplyDate": None,
@@ -321,7 +314,6 @@ class Comment(Model):
         "parentId": None,
         "text": None,
     }
-    service = None  # type: CommentsAPI
 
     @property
     def uid(self):
@@ -372,9 +364,6 @@ class FileBlob(Blob):
     with the `with` statement.
     """
 
-    # File descriptor
-    fd = None  # type: Optional[BinaryIO]
-
     def __init__(self, path, **kwargs):
         # type: (Text, Any) -> None
         """
@@ -382,6 +371,7 @@ class FileBlob(Blob):
         :param **kwargs: named attributes
         """
         super(FileBlob, self).__init__(**kwargs)
+        self.fd = None  # type: Optional[BinaryIO]
         self.path = path
         self.name = os.path.basename(self.path)
         self.size = os.path.getsize(self.path)
@@ -411,12 +401,11 @@ class FileBlob(Blob):
 class Directory(Model):
     """ Directory. """
 
-    _valid_properties = {
+    __slots__ = {
         "directoryName": None,
         "entries": [],
-        "entity-type": "directory",
+        "entity_type": "directory",
     }
-    service = None  # type: DirectoriesAPI
 
     @property
     def uid(self):
@@ -466,12 +455,11 @@ class Directory(Model):
 class DirectoryEntry(Model):
     """ Directory entry. """
 
-    _valid_properties = {
+    __slots__ = {
         "directoryName": None,
-        "entity-type": "directoryEntry",
+        "entity_type": "directoryEntry",
         "properties": {},
     }
-    service = None  # type: DirectoriesAPI
 
     @property
     def uid(self):
@@ -492,10 +480,10 @@ class DirectoryEntry(Model):
 class Document(RefreshableModel):
     """ Document. """
 
-    _valid_properties = {
+    __slots__ = {
         "changeToken": None,
         "contextParameters": {},
-        "entity-type": "document",
+        "entity_type": "document",
         "facets": [],
         "isCheckedOut": False,
         "isProxy": False,
@@ -513,7 +501,6 @@ class Document(RefreshableModel):
         "uid": None,
         "versionLabel": None,
     }
-    service = None  # type: DocumentsAPI
 
     @property
     def workflows(self):
@@ -672,14 +659,13 @@ class Document(RefreshableModel):
 class Group(Model):
     """ User group. """
 
-    _valid_properties = {
-        "entity-type": "group",
+    __slots__ = {
+        "entity_type": "group",
         "grouplabel": None,
         "groupname": None,
         "memberGroups": [],
         "memberUsers": [],
     }
-    service = None  # type: GroupsAPI
 
     @property
     def uid(self):
@@ -695,14 +681,13 @@ class Group(Model):
 class Operation(Model):
     """ Automation operation. """
 
-    _valid_properties = {
+    __slots__ = {
         "command": None,
         "context": None,
         "input_obj": None,
         "params": {},
         "progress": 0,
     }
-    service = None  # type: OperationsAPI
 
     def execute(self, **kwargs):
         # type: (Any) -> Any
@@ -713,13 +698,13 @@ class Operation(Model):
 class Task(RefreshableModel):
     """ Workflow task. """
 
-    _valid_properties = {
+    __slots__ = {
         "actors": [],
         "comments": [],
         "created": None,
         "directive": None,
         "dueDate": None,
-        "entity-type": "task",
+        "entity_type": "task",
         "id": None,
         "name": None,
         "nodeName": None,
@@ -730,7 +715,6 @@ class Task(RefreshableModel):
         "workflowInstanceId": None,
         "workflowModelName": None,
     }
-    service = None  # type: TasksAPI
 
     @property
     def uid(self):
@@ -761,15 +745,14 @@ class Task(RefreshableModel):
 class User(RefreshableModel):
     """ User. """
 
-    _valid_properties = {
-        "entity-type": "user",
+    __slots__ = {
+        "entity_type": "user",
         "extendedGroups": [],
         "id": None,
         "isAdministrator": False,
         "isAnonymous": False,
         "properties": {},
     }
-    service = None  # type: UsersAPI
 
     @property
     def uid(self):
@@ -795,9 +778,9 @@ class User(RefreshableModel):
 class Workflow(Model):
     """ Workflow. """
 
-    _valid_properties = {
+    __slots__ = {
         "attachedDocumentIds": [],
-        "entity-type": "workflow",
+        "entity_type": "workflow",
         "graphResource": None,
         "id": None,
         "initiator": None,
@@ -807,7 +790,6 @@ class Workflow(Model):
         "variables": {},
         "workflowModelName": None,
     }
-    service = None  # type: WorkflowsAPI
 
     @property
     def tasks(self):
